/*
 * Serial_interrupt_problem_eboard.c
 *
 * Created: 06/08/2015 12:31:10
 *  Author: Duresh Asanka
 * Created to solve the problem in: http://www.edaboard.com/thread342009.html
 *
 */ 

#define F_CPU 11059200

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include "my_usart.h"


#define RX_BUFFER_SIZE 30			// Receive buffer size
uint8_t RX_buffer[RX_BUFFER_SIZE];	// Receive buffer
uint8_t buff_pos_store = 0;			// buffer position to store next coming byte
uint8_t buff_pos_read = 0;			// buffer position to read
//bool buff_overflow = false;

volatile int sr_flag=0;

ISR(USART_RXC_vect)
{
	/* check whether the user has read all the data on the buffer before storing new coming byte. Else set a flag */
	RX_buffer[buff_pos_store++] = UDR;
	
	sr_flag=1;

	/* reset the buffer position variable to store next coming data on to the fist position */
	if(buff_pos_store >= RX_BUFFER_SIZE)
	{
		buff_pos_store = 0;
	}
}


int main (void)
{
	/* these are flags to indicate that we have got 8,0,2,1 */
	bool got_8 = false;
	bool got_0 = false;
	bool got_2 = false;
	bool got_1 = false;
	
	/*set the buffer to null */
	memset(RX_buffer, NULL, RX_BUFFER_SIZE);
	
	DDRC=0xFF;
	PORTC=0x00;
	
	USARTInit(71);	
	
	while(1) // Loop forever
	{
		PORTC=0xAA;
		_delay_ms(50);
		PORTC=0x55;
		_delay_ms(50);
		
		if(sr_flag==1)
		{
			/* read the buffer until you come to last-data-stored position */
			while(buff_pos_read != buff_pos_store)
			{
				/* your compare function can be implemented here */
				//USARTWriteChar(RX_buffer[buff_pos_read]);
				
				/* check for 8 */
				if (got_8)
				{
					/* check for 0 */
					if (got_0)
					{
						/* check for 2 */
						if (got_2)
						{
							/*check for 1 */
							if (got_1)
							{
								/* now we have found a complete packet. need to decode from the character */
								switch (RX_buffer[buff_pos_read])
								{
									case 'A':
									USARTWriteChar('a');
									break;
									case 'B':
									USARTWriteChar('b');
									break;
									case 'C':
									USARTWriteChar('c');
									break;
									case 'D':
									USARTWriteChar('d');
									break;
									case 'E':
									USARTWriteChar('e');
									break;
									case 'F':
									USARTWriteChar('f');
									break;
									case 'G':
									USARTWriteChar('g');
									break;
									case 'H':
									USARTWriteChar('h');
									break;
									case 'I':
									USARTWriteChar('i');
									break;
									case 'J':
									USARTWriteChar('j');
									break;
									case 'K':
									USARTWriteChar('k');
									break;
									case 'L':
									USARTWriteChar('l');
									break;
									case 'M':
									USARTWriteChar('m');
									break;
									case 'N':
									USARTWriteChar('n');
									break;
									case 'O':
									USARTWriteChar('o');
									break;
									case 'P':
									USARTWriteChar('p');
									break;
									default:
									USARTWriteChar('?');
									break;
								}
								/* after one successful packet we need to reset all flags */
								got_8 = false;
								got_0 = false;
								got_2 = false;
								got_1 = false;
							} 
							else if (RX_buffer[buff_pos_read] == '1')
							{
								/* we got 1, so set the flag for next round */
								got_1 = true;
							}
						}
						else if (RX_buffer[buff_pos_read] == '2')
						{
							/* we got 2, so set the flag for next round */
							got_2 = true;
						}
					}
					else if (RX_buffer[buff_pos_read] == '0')
					{
						/* we got 0, so set the flag for next round */
						got_0 = true;
					}
				}
				else if (RX_buffer[buff_pos_read] == '8')
				{
					/* reset all other flags, so that we can ensure we read one packet */
					got_8 = true;
					got_0 = false;
					got_2 = false;
					got_1 = false;
				}
				
				/* we need to set read value to invalid character to avoid read the same character */
				RX_buffer[buff_pos_read] = NULL;
				
				/* increment the read position */
				buff_pos_read++;
				/* if next read-position is outside of the buffer, then reset the positon. */
				if(buff_pos_read >= RX_BUFFER_SIZE)
				{
					buff_pos_read = 0;
				}
			}
			/* clear the flag */
			sr_flag = 0;
		}
	}
}

